@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager_config.c,v $
- * $Date: 2012/04/11 06:00:29 $
- * $Revision: 1.24.4.1 $
+ * $Date: 2012/06/06 14:04:51 $
+ * $Revision: 1.24.4.3 $
  * $Author: bester $
  */
 
@@ -801,6 +801,16 @@ globus_gram_job_manager_config_destroy(
     {
         free(config->globus_version);
     }
+    if (config->usage_targets)
+    {
+        free(config->usage_targets);
+    }
+    /*
+    if (config->log_pattern)
+    {
+        free(config->log_pattern);
+    }
+    */
     if (config->subject)
     {
         free(config->subject);
@@ -817,6 +827,18 @@ globus_gram_job_manager_config_destroy(
     {
         free(config->hostname);
     }
+    if (config->short_hostname)
+    {
+        free(config->short_hostname);
+    }
+    if (config->service_tag)
+    {
+        free(config->service_tag);
+    }
+    if (config->job_dir_home)
+    {
+        free(config->job_dir_home);
+    }
 }
 /* globus_gram_job_manager_config_destroy() */
 
