@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/globus_gram_job_manager_query.c,v $
- * $Date: 2012/04/11 12:53:59 $
- * $Revision: 1.28.8.4 $
+ * $Date: 2012/05/24 12:32:44 $
+ * $Revision: 1.28.8.7 $
  * $Author: bester $
  */
 #include "globus_gram_job_manager.h"
@@ -499,8 +499,8 @@ globus_l_gram_job_manager_query_reply(
 
         replystring = globus_gram_prepare_log_string((char *) reply);
 
-        globus_gram_job_manager_request_log(
-            request,
+        globus_gram_job_manager_log(
+            manager,
             GLOBUS_GRAM_JOB_MANAGER_LOG_TRACE,
             "event=gram.query.info "
             "%s%s%s"
@@ -2067,30 +2067,38 @@ globus_l_gram_stdio_update_signal(
     new_unevaluated_rsl = globus_gram_job_manager_rsl_merge(
             original_rsl,
             rsl);
-    globus_rsl_free_recursive(original_rsl);
+    if (new_unevaluated_rsl == NULL)
+    {
+        rc = GLOBUS_GRAM_PROTOCOL_ERROR_BAD_RSL;
+
+        goto free_original_rsl_out;
+    }
     new_rsl_spec = globus_rsl_unparse(new_unevaluated_rsl);
     if (new_rsl_spec == NULL)
     {
-        globus_rsl_free_recursive(new_unevaluated_rsl);
-
         rc = GLOBUS_GRAM_PROTOCOL_ERROR_BAD_RSL;
 
-        goto free_rsl_out;
+        goto free_new_unevaluated_rsl_out;
     }
     free(request->rsl_spec);
     request->rsl_spec = new_rsl_spec;
+    new_rsl_spec = NULL;
 
     rc = globus_rsl_eval(rsl, &request->symbol_table);
     if(rc != GLOBUS_SUCCESS)
     {
         rc = GLOBUS_GRAM_PROTOCOL_ERROR_RSL_EVALUATION_FAILED;
 
-        goto free_rsl_out;
+        goto free_new_unevaluated_rsl_out;
     }
 
     rc = globus_i_gram_request_stdio_update(
             request,
             rsl);
+free_new_unevaluated_rsl_out:
+    globus_rsl_free_recursive(new_unevaluated_rsl);
+free_original_rsl_out:
+    globus_rsl_free_recursive(original_rsl);
 free_rsl_out:
     globus_rsl_free_recursive(rsl);
 error_out:
