@@ -30,6 +30,10 @@ typedef struct globus_gram_seg_resume_s
 }
 globus_gram_seg_resume_t;
 
+static globus_bool_t globus_l_condor_regexes_compiled = GLOBUS_FALSE;
+static regex_t globus_l_condor_outer_re;
+static regex_t globus_l_condor_inner_re;
+
 globus_result_t
 globus_l_gram_seg_event_callback(
     void *                              user_arg,
@@ -991,6 +995,7 @@ globus_l_gram_condor_poll_callback(
         poll_time = last_poll_time;
     }
 
+
     for (ref = globus_hashtable_first(&manager->request_hash);
          ref != NULL;
          ref = globus_hashtable_next(&manager->request_hash))
@@ -1084,6 +1089,12 @@ read_failed:
     {
         manager->seg_last_timestamp = poll_time;
     }
+    if (globus_l_condor_regexes_compiled)
+    {
+        regfree(&globus_l_condor_outer_re);
+        regfree(&globus_l_condor_inner_re);
+        globus_l_condor_regexes_compiled = GLOBUS_FALSE;
+    }
     GlobusGramJobManagerUnlock(manager);
     globus_gram_job_manager_log(
             manager,
@@ -1124,8 +1135,6 @@ globus_l_condor_parse_log(
     globus_gram_job_manager_ref_t *     ref,
     globus_fifo_t *                     events)
 {
-    static int                          once = 0;
-    static regex_t                      outer_re, inner_re;
     regmatch_t                          matches[8];
     const char *                        p;
     int                                 event_type_number;
@@ -1189,39 +1198,37 @@ globus_l_condor_parse_log(
         } r;
     } pu;
 
-    if (!once)
-    {
-        once = 1;
+    p = data;
+    parsed_length = 0;
 
-        rc = regcomp(&outer_re,
-            "(<c>((<[^/]|</[^c]>|[^<])*)</c>)",
+    if (!globus_l_condor_regexes_compiled)
+    {
+        rc = regcomp(&globus_l_condor_outer_re,
+            "^([[:space:]]*<c>((<[^/]|</[^c]>|</[^>]{2,}>|[^<])*)</c>[[:space:]]*)",
             REG_EXTENDED);
 
         assert (rc == 0);
 
-        rc = regcomp(&inner_re,
+        rc = regcomp(&globus_l_condor_inner_re,
             "^([[:space:]]*"
             "<a n=\"([[:alpha:]]+)\">[[:space:]]*"
-            "(<(b) v=\"([tf])\"/>|<([sir])>([^<]*)</[sir]>)"
+            "(<(b) v=\"([tf])\"/>|<([sire])>([^<]*)</[sire]>)"
             "</a>[[:space:]]*)",
             REG_EXTENDED);
 
         assert(rc == 0);
+        globus_l_condor_regexes_compiled = GLOBUS_TRUE;
     }
-
-    p = data + ref->seg_last_size;
-    parsed_length = ref->seg_last_size;
-
     while ((rc = regexec(
-                &outer_re, p, (int) (sizeof(matches)/sizeof(matches[0])),
+                &globus_l_condor_outer_re, p, (int) (sizeof(matches)/sizeof(matches[0])),
                 matches, 0)) == 0)
     {
-        const char * e = p + matches[1].rm_eo;
+        const char * e = p + matches[0].rm_eo;
         regoff_t event_length = matches[0].rm_eo - matches[0].rm_so;
 
         p = p + matches[2].rm_so;
 
-        while ((rc = regexec(&inner_re, p,
+        while ((rc = regexec(&globus_l_condor_inner_re, p,
                     (int) (sizeof(matches)/sizeof(matches[0])),
                     matches, 0)) == 0)
         {
@@ -1314,6 +1321,13 @@ globus_l_condor_parse_log(
                     pu.type = CONDOR_REAL;
                     sscanf(p + matches[7].rm_so, "%f", &pu.r.r);
                 }
+                else if (strncmp(match, "e", matchlen) == 0)
+                {
+                    /* ? */
+                    pu.type = CONDOR_STRING;
+                    pu.s.s = p + matches[7].rm_so;
+                    pu.s.len = (size_t) (matches[7].rm_eo - matches[7].rm_so);
+                }
             }
             switch (condor_attr)
             {
@@ -1415,7 +1429,7 @@ globus_l_condor_parse_log(
             ref->seg_last_timestamp = event->timestamp;
         }
     }
-    ref->seg_last_size = parsed_length;
+    ref->seg_last_size += parsed_length;
     return 0;
 }
 /* globus_l_condor_parse_log() */
@@ -1541,10 +1555,11 @@ globus_l_condor_read_log(
 
     {
         ssize_t read_res;
-        size_t amt_to_read = st.st_size;
+        size_t amt_to_read = st.st_size - last_size;
         size_t amt_read = 0;
+        off_t off_rc;
 
-        condor_log_data = malloc((size_t) st.st_size + 1);
+        condor_log_data = malloc(amt_to_read + 1);
         if (condor_log_data == NULL)
         {
             globus_gram_job_manager_log(
@@ -1565,7 +1580,30 @@ globus_l_condor_read_log(
             rc = GLOBUS_GRAM_PROTOCOL_ERROR_MALLOC_FAILED;
             goto malloc_data_failed;
         }
-        condor_log_data[(size_t) st.st_size] = 0;
+
+        condor_log_data[amt_to_read] = 0;
+
+        off_rc = lseek(condor_log_fd, (off_t) last_size, SEEK_SET);
+        if (off_rc < 0)
+        {
+            globus_gram_job_manager_log(
+                    manager,
+                    GLOBUS_GRAM_JOB_MANAGER_LOG_WARN,
+                    "event=gram.condor_poll.info "
+                    "level=WARN "
+                    "message=\"%s\" "
+                    "filename=\"%s\" "
+                    "size=%llu "
+                    "errno=%d "
+                    "reason=%s\n",
+                    "Error seeking in condor log",
+                    path,
+                    (unsigned long long) st.st_size,
+                    errno,
+                    strerror(errno));
+            rc = GLOBUS_GRAM_PROTOCOL_ERROR_TEMP_SCRIPT_FILE_FAILED;
+            goto seek_failed;
+        }
 
         while (amt_to_read > amt_read)
         {
@@ -1617,6 +1655,7 @@ globus_l_condor_read_log(
     if (rc != GLOBUS_SUCCESS)
     {
 read_failed:
+seek_failed:
         free(condor_log_data);
     }
 malloc_data_failed:
