@@ -21,8 +21,8 @@
  * CVS Information:
  * 
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/main.c,v $
- * $Date: 2012/03/14 18:42:28 $
- * $Revision: 1.41.2.3 $
+ * $Date: 2012/07/17 18:56:54 $
+ * $Revision: 1.41.2.5 $
  * $Author: bester $
  */
 
@@ -518,6 +518,19 @@ main(
                     globus_l_gram_lockcheck,
                     &manager);
             }
+
+            {
+                globus_reltime_t        idlescript_period;
+
+                GlobusTimeReltimeSet(idlescript_period, 60, 0);
+
+                rc = globus_callback_register_periodic(
+                    &manager.idle_script_handle,
+                    NULL,
+                    &idlescript_period,
+                    globus_gram_script_close_idle,
+                    &manager);
+            }
         }
         else if (http_body_fd >= 0)
         {
@@ -589,7 +602,12 @@ main(
         act.sa_handler = SIG_IGN;
         sigemptyset(&act.sa_mask);
         sigaddset(&act.sa_mask, SIGCHLD);
+#ifdef SA_NOCLDWAIT
         act.sa_flags = SA_NOCLDWAIT;
+#else
+        /* This may leave zombies running on non-POSIX systems like Hurd */
+        act.sa_flags = 0;
+#endif
         sigaction(SIGCHLD, &act, NULL);
     }
 
@@ -628,6 +646,10 @@ main(
     {
         globus_callback_unregister(manager.lockcheck_handle, NULL, NULL, NULL);
     }
+    if (manager.idle_script_handle != GLOBUS_NULL_HANDLE)
+    {
+        globus_callback_unregister(manager.idle_script_handle, NULL, NULL, NULL);
+    }
     GlobusGramJobManagerUnlock(&manager);
 
     globus_gram_job_manager_log(
