@@ -20,8 +20,8 @@
  *
  * CVS Information:
  * $Source: /home/globdev/CVS/globus-packages/gram/jobmanager/source/startup_socket.c,v $
- * $Date: 2012/04/11 12:53:59 $
- * $Revision: 1.19.2.4 $
+ * $Date: 2012/05/16 19:24:41 $
+ * $Revision: 1.19.2.7 $
  * $Author: bester $
  */
 #include "globus_common.h"
@@ -1217,7 +1217,7 @@ globus_l_gram_startup_socket_callback(
     int                                 response_fd = -1;
     int                                 rc;
     gss_ctx_id_t                        context;
-    char *                              contact;
+    char *                              contact = NULL;
     int                                 job_state_mask;
     globus_gram_jobmanager_request_t *  request;
     OM_uint32                           major_status, minor_status;
@@ -1982,9 +1982,12 @@ globus_l_gram_startup_socket_callback(
         }
 
 update_cred_failed:
-        free(contact);
-
 request_load_failed:
+        if (contact)
+        {
+            free(contact);
+        }
+
         if (old_job_contact != NULL)
         {
             free(old_job_contact);
@@ -2156,7 +2159,7 @@ globus_l_create_starter_connect_socket(
     {
         rc = GLOBUS_GRAM_PROTOCOL_ERROR_NO_RESOURCES;
         save_errno = errno;
-        msg = "Error making datagram connecting to Job Manager";
+        msg = "Error making datagram connection to Job Manager";
 
 setsockopt_failed:
         close(sock);
@@ -2171,11 +2174,13 @@ socket_failed:
                 "errno=%d "
                 "msg=\"%s\" "
                 "reason=\"%s\" "
+                "path=\"%s\" "
                 "\n",
                 -rc,
                 save_errno,
-                "Error creating datagram socket",
-                strerror(save_errno));
+                msg,
+                strerror(save_errno),
+                addr.sun_path);
     }
     *sockptr = sock;
     return rc;
